from enum import Enum


class TestResultEnum(str, Enum):
    CANCELLED = "cancelled"
    ERRORED = "errored"
    FAILED = "failed"
    PASSED = "passed"
    PENDING = "pending"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ConversationNestedTokenCost")


@_attrs_define
class ConversationNestedTokenCost:
    """
    Attributes:
        id (str):
        input_tokens (int):
        output_tokens (int):
        cost_amount (int):
        cost_currency (str):
    """

    id: str
    input_tokens: int
    output_tokens: int
    cost_amount: int
    cost_currency: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        input_tokens = self.input_tokens

        output_tokens = self.output_tokens

        cost_amount = self.cost_amount

        cost_currency = self.cost_currency

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "input_tokens": input_tokens,
                "output_tokens": output_tokens,
                "cost_amount": cost_amount,
                "cost_currency": cost_currency,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        input_tokens = d.pop("input_tokens")

        output_tokens = d.pop("output_tokens")

        cost_amount = d.pop("cost_amount")

        cost_currency = d.pop("cost_currency")

        conversation_nested_token_cost = cls(
            id=id,
            input_tokens=input_tokens,
            output_tokens=output_tokens,
            cost_amount=cost_amount,
            cost_currency=cost_currency,
        )

        conversation_nested_token_cost.additional_properties = d
        return conversation_nested_token_cost

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import os
import pymupdf
import docx2txt

from typing import List, Iterable, Optional
import re


def extract_text(file_path):
    file_extension = os.path.splitext(file_path)[1].lower()

    if file_extension == ".pdf":
        return extract_pdf_text(file_path)
    elif file_extension == ".docx":
        return extract_docx_text(file_path)
    elif file_extension in [".txt", ".md"]:
        return extract_text_file(file_path)
    else:
        raise ValueError(f"Unsupported file type: {file_extension}")


def extract_pdf_text(file_path):
    doc = pymupdf.open(file_path)
    text = ""
    for page in doc:
        text += page.get_text()
    return text


def extract_docx_text(file_path):
    text = docx2txt.process(file_path)
    return text


def extract_text_file(file_path):
    doc = pymupdf.open(file_path, filetype="txt")
    text = ""
    for page in doc:
        text += page.get_text()
    return text


class TextSplitter:
    def __init__(
        self,
        chunk_size: int,
        chunk_overlap: int,
        separators: List[str] = ["\n\n", "\n", " ", ""],
    ):
        self._chunk_size = chunk_size
        self._chunk_overlap = chunk_overlap
        self._separators = separators
        self._is_separator_regex = False
        self._keep_separator = False
        self._length_function = len

    def split_text(self, text: str) -> List[str]:
        return self._split_text(text, self._separators)

    def _split_text(self, text: str, separators: List[str]) -> List[str]:
        """Split incoming text and return chunks."""
        final_chunks = []
        # Get appropriate separator to use
        separator = separators[-1]
        new_separators = []
        for i, _s in enumerate(separators):
            _separator = _s if self._is_separator_regex else re.escape(_s)
            if _s == "":
                separator = _s
                break
            if re.search(_separator, text):
                separator = _s
                new_separators = separators[i + 1 :]
                break

        _separator = separator if self._is_separator_regex else re.escape(separator)
        splits = self._split_text_with_regex(text, _separator, self._keep_separator)

        # Now go merging things, recursively splitting longer texts.
        _good_splits = []
        _separator = "" if self._keep_separator else separator
        for s in splits:
            if self._length_function(s) < self._chunk_size:
                _good_splits.append(s)
            else:
                if _good_splits:
                    merged_text = self._merge_splits(_good_splits, _separator)
                    final_chunks.extend(merged_text)
                    _good_splits = []
                if not new_separators:
                    final_chunks.append(s)
                else:
                    other_info = self._split_text(s, new_separators)
                    final_chunks.extend(other_info)
        if _good_splits:
            merged_text = self._merge_splits(_good_splits, _separator)
            final_chunks.extend(merged_text)
        return final_chunks

    @staticmethod
    def _split_text_with_regex(
        text: str, separator: str, keep_separator: bool
    ) -> List[str]:
        if separator:
            if keep_separator:
                splits = re.split(f"({separator})", text)
                return [
                    splits[i] + splits[i + 1] for i in range(0, len(splits) - 1, 2)
                ] + ([splits[-1]] if len(splits) % 2 == 1 else [])
            else:
                return re.split(separator, text)
        else:
            return list(text)

    def _merge_splits(self, splits: Iterable[str], separator: str) -> List[str]:

        separator_len = self._length_function(separator)

        docs = []
        current_doc: List[str] = []
        total = 0
        for d in splits:
            _len = self._length_function(d)
            if (
                total + _len + (separator_len if len(current_doc) > 0 else 0)
                > self._chunk_size
            ):
                if total > self._chunk_size:
                    logger.warning(
                        f"Created a chunk of size {total}, "
                        f"which is longer than the specified {self._chunk_size}"
                    )
                if len(current_doc) > 0:
                    doc = self._join_docs(current_doc, separator)
                    if doc is not None:
                        docs.append(doc)

                    while total > self._chunk_overlap or (
                        total + _len + (separator_len if len(current_doc) > 0 else 0)
                        > self._chunk_size
                        and total > 0
                    ):
                        total -= self._length_function(current_doc[0]) + (
                            separator_len if len(current_doc) > 1 else 0
                        )
                        current_doc = current_doc[1:]
            current_doc.append(d)
            total += _len + (separator_len if len(current_doc) > 1 else 0)
        doc = self._join_docs(current_doc, separator)
        if doc is not None:
            docs.append(doc)
        return docs

    def _join_docs(self, docs: List[str], separator: str) -> Optional[str]:
        text = separator.join(docs)
        text = text.strip()
        if text == "":
            return None
        return text



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="APISchemaTestRunConversationCounts")


@_attrs_define
class APISchemaTestRunConversationCounts:
    """
    Attributes:
        total_conversations (int):
        result_passed (int):
        result_failed (int):
        result_errored (int):
        result_pending (int):
        result_cancelled (int):
        status_completed (int):
        status_failed (int):
        status_running (int):
        status_pending (int):
        status_paused (int):
        total_score (Union[None, Unset, float]):
    """

    total_conversations: int
    result_passed: int
    result_failed: int
    result_errored: int
    result_pending: int
    result_cancelled: int
    status_completed: int
    status_failed: int
    status_running: int
    status_pending: int
    status_paused: int
    total_score: Union[None, Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total_conversations = self.total_conversations

        result_passed = self.result_passed

        result_failed = self.result_failed

        result_errored = self.result_errored

        result_pending = self.result_pending

        result_cancelled = self.result_cancelled

        status_completed = self.status_completed

        status_failed = self.status_failed

        status_running = self.status_running

        status_pending = self.status_pending

        status_paused = self.status_paused

        total_score: Union[None, Unset, float]
        if isinstance(self.total_score, Unset):
            total_score = UNSET
        else:
            total_score = self.total_score

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total_conversations": total_conversations,
                "result_passed": result_passed,
                "result_failed": result_failed,
                "result_errored": result_errored,
                "result_pending": result_pending,
                "result_cancelled": result_cancelled,
                "status_completed": status_completed,
                "status_failed": status_failed,
                "status_running": status_running,
                "status_pending": status_pending,
                "status_paused": status_paused,
            }
        )
        if total_score is not UNSET:
            field_dict["total_score"] = total_score

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total_conversations = d.pop("total_conversations")

        result_passed = d.pop("result_passed")

        result_failed = d.pop("result_failed")

        result_errored = d.pop("result_errored")

        result_pending = d.pop("result_pending")

        result_cancelled = d.pop("result_cancelled")

        status_completed = d.pop("status_completed")

        status_failed = d.pop("status_failed")

        status_running = d.pop("status_running")

        status_pending = d.pop("status_pending")

        status_paused = d.pop("status_paused")

        def _parse_total_score(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        total_score = _parse_total_score(d.pop("total_score", UNSET))

        api_schema_test_run_conversation_counts = cls(
            total_conversations=total_conversations,
            result_passed=result_passed,
            result_failed=result_failed,
            result_errored=result_errored,
            result_pending=result_pending,
            result_cancelled=result_cancelled,
            status_completed=status_completed,
            status_failed=status_failed,
            status_running=status_running,
            status_pending=status_pending,
            status_paused=status_paused,
            total_score=total_score,
        )

        api_schema_test_run_conversation_counts.additional_properties = d
        return api_schema_test_run_conversation_counts

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pymupdf  # PyMuPDF
from docx import Document
from PIL import Image
import io
import os
import requests


def extract_images_and_convert_text(document_path):
    """
    Extracts images from a PDF or DOCX document and converts them into text using an LLM.

    Args:
        document_path (str): The path to the PDF or DOCX document.

    Returns:
        dict: A dictionary where keys are image file names and values are the extracted text.
    """
    extracted_text = {}
    file_extension = os.path.splitext(document_path)[1].lower()

    if file_extension == ".pdf":
        extracted_text = _process_pdf(document_path)
    else:
        raise ValueError("Unsupported file type. Only PDF and DOCX are supported.")

    return extracted_text


def _process_pdf(pdf_path):
    """
    Extracts images from a PDF and converts them into text using an LLM.

    Args:
        pdf_path (str): The path to the PDF file.

    Returns:
        dict: A dictionary where keys are image identifiers and values are the extracted text.
    """
    extracted = {}
    doc = pymupdf.open(pdf_path)
    for page_num in range(len(doc)):
        page = doc.load_page(page_num)
        image_list = page.get_images(full=True)
        for img_index, img in enumerate(image_list):
            xref = img[0]
            base_image = doc.extract_image(xref)
            image_bytes = base_image["image"]
            image_ext = base_image["ext"]
            image = Image.open(io.BytesIO(image_bytes))

            # Convert image to bytes for API
            buffered = io.BytesIO()
            image.save(buffered, format=image_ext.upper())
            img_bytes = buffered.getvalue()

            # Query LLM
            text = _query_llm(img_bytes)
            image_id = f"page_{page_num+1}_image_{img_index+1}.{image_ext}"
            extracted[image_id] = text

    return extracted


def _query_llm(image_bytes):
    """
    Sends the image bytes to an LLM API to convert the image into text.

    Args:
        image_bytes (bytes): The image data in bytes.

    Returns:
        str: The text extracted from the image.
    """
    api_url = "https://api.your-llm-service.com/convert-image"
    headers = {
        "Authorization": "Bearer YOUR_API_TOKEN",
        "Content-Type": "application/octet-stream",
    }
    response = requests.post(api_url, headers=headers, data=image_bytes)

    if response.status_code == 200:
        return response.json().get("text", "")
    else:
        return ""



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.test_result_enum import TestResultEnum
from ..models.test_status_enum import TestStatusEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="APISchemaAgentTarget")


@_attrs_define
class APISchemaAgentTarget:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        identifier (str):
        role (str):
        language (str):
        name (Union[None, Unset, str]):
        description (Union[None, Unset, str]):
        industry (Union[None, Unset, str]):
        url (Union[None, Unset, str]):
        last_test_run_id (Union[None, Unset, str]):
        last_test_run_at (Union[None, Unset, datetime.datetime]):
        last_test_run_status (Union[None, TestStatusEnum, Unset]):
        last_test_run_result (Union[None, TestResultEnum, Unset]):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    identifier: str
    role: str
    language: str
    name: Union[None, Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    industry: Union[None, Unset, str] = UNSET
    url: Union[None, Unset, str] = UNSET
    last_test_run_id: Union[None, Unset, str] = UNSET
    last_test_run_at: Union[None, Unset, datetime.datetime] = UNSET
    last_test_run_status: Union[None, TestStatusEnum, Unset] = UNSET
    last_test_run_result: Union[None, TestResultEnum, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        identifier = self.identifier

        role = self.role

        language = self.language

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        industry: Union[None, Unset, str]
        if isinstance(self.industry, Unset):
            industry = UNSET
        else:
            industry = self.industry

        url: Union[None, Unset, str]
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        last_test_run_id: Union[None, Unset, str]
        if isinstance(self.last_test_run_id, Unset):
            last_test_run_id = UNSET
        else:
            last_test_run_id = self.last_test_run_id

        last_test_run_at: Union[None, Unset, str]
        if isinstance(self.last_test_run_at, Unset):
            last_test_run_at = UNSET
        elif isinstance(self.last_test_run_at, datetime.datetime):
            last_test_run_at = self.last_test_run_at.isoformat()
        else:
            last_test_run_at = self.last_test_run_at

        last_test_run_status: Union[None, Unset, str]
        if isinstance(self.last_test_run_status, Unset):
            last_test_run_status = UNSET
        elif isinstance(self.last_test_run_status, TestStatusEnum):
            last_test_run_status = self.last_test_run_status.value
        else:
            last_test_run_status = self.last_test_run_status

        last_test_run_result: Union[None, Unset, str]
        if isinstance(self.last_test_run_result, Unset):
            last_test_run_result = UNSET
        elif isinstance(self.last_test_run_result, TestResultEnum):
            last_test_run_result = self.last_test_run_result.value
        else:
            last_test_run_result = self.last_test_run_result

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "identifier": identifier,
                "role": role,
                "language": language,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if industry is not UNSET:
            field_dict["industry"] = industry
        if url is not UNSET:
            field_dict["url"] = url
        if last_test_run_id is not UNSET:
            field_dict["last_test_run_id"] = last_test_run_id
        if last_test_run_at is not UNSET:
            field_dict["last_test_run_at"] = last_test_run_at
        if last_test_run_status is not UNSET:
            field_dict["last_test_run_status"] = last_test_run_status
        if last_test_run_result is not UNSET:
            field_dict["last_test_run_result"] = last_test_run_result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        identifier = d.pop("identifier")

        role = d.pop("role")

        language = d.pop("language")

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_industry(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        industry = _parse_industry(d.pop("industry", UNSET))

        def _parse_url(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        url = _parse_url(d.pop("url", UNSET))

        def _parse_last_test_run_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_test_run_id = _parse_last_test_run_id(d.pop("last_test_run_id", UNSET))

        def _parse_last_test_run_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_test_run_at_type_0 = isoparse(data)

                return last_test_run_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        last_test_run_at = _parse_last_test_run_at(d.pop("last_test_run_at", UNSET))

        def _parse_last_test_run_status(data: object) -> Union[None, TestStatusEnum, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_test_run_status_type_0 = TestStatusEnum(data)

                return last_test_run_status_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TestStatusEnum, Unset], data)

        last_test_run_status = _parse_last_test_run_status(d.pop("last_test_run_status", UNSET))

        def _parse_last_test_run_result(data: object) -> Union[None, TestResultEnum, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_test_run_result_type_0 = TestResultEnum(data)

                return last_test_run_result_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TestResultEnum, Unset], data)

        last_test_run_result = _parse_last_test_run_result(d.pop("last_test_run_result", UNSET))

        api_schema_agent_target = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            identifier=identifier,
            role=role,
            language=language,
            name=name,
            description=description,
            industry=industry,
            url=url,
            last_test_run_id=last_test_run_id,
            last_test_run_at=last_test_run_at,
            last_test_run_status=last_test_run_status,
            last_test_run_result=last_test_run_result,
        )

        api_schema_agent_target.additional_properties = d
        return api_schema_agent_target

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import NamedTuple, Optional
from maihem.api_client.maihem_client.models.conversation_nested import (
    ConversationNested,
)


class ConversationTurnCreateResponse(NamedTuple):
    conversation: ConversationNested
    turn_id: Optional[str] = None



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.conversation_nested_evaluation import ConversationNestedEvaluation
    from ..models.conversation_nested_message import ConversationNestedMessage


T = TypeVar("T", bound="ConversationNestedTurn")


@_attrs_define
class ConversationNestedTurn:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        evaluations (Union[Unset, List['ConversationNestedEvaluation']]):
        conversation_messages (Union[Unset, List['ConversationNestedMessage']]):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    evaluations: Union[Unset, List["ConversationNestedEvaluation"]] = UNSET
    conversation_messages: Union[Unset, List["ConversationNestedMessage"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        evaluations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.evaluations, Unset):
            evaluations = []
            for evaluations_item_data in self.evaluations:
                evaluations_item = evaluations_item_data.to_dict()
                evaluations.append(evaluations_item)

        conversation_messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.conversation_messages, Unset):
            conversation_messages = []
            for conversation_messages_item_data in self.conversation_messages:
                conversation_messages_item = conversation_messages_item_data.to_dict()
                conversation_messages.append(conversation_messages_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if evaluations is not UNSET:
            field_dict["evaluations"] = evaluations
        if conversation_messages is not UNSET:
            field_dict["conversation_messages"] = conversation_messages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.conversation_nested_evaluation import ConversationNestedEvaluation
        from ..models.conversation_nested_message import ConversationNestedMessage

        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        evaluations = []
        _evaluations = d.pop("evaluations", UNSET)
        for evaluations_item_data in _evaluations or []:
            evaluations_item = ConversationNestedEvaluation.from_dict(evaluations_item_data)

            evaluations.append(evaluations_item)

        conversation_messages = []
        _conversation_messages = d.pop("conversation_messages", UNSET)
        for conversation_messages_item_data in _conversation_messages or []:
            conversation_messages_item = ConversationNestedMessage.from_dict(conversation_messages_item_data)

            conversation_messages.append(conversation_messages_item)

        conversation_nested_turn = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            evaluations=evaluations,
            conversation_messages=conversation_messages,
        )

        conversation_nested_turn.additional_properties = d
        return conversation_nested_turn

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.error_codes import ErrorCodes
from ..types import UNSET, Unset

T = TypeVar("T", bound="ErrorResponseError")


@_attrs_define
class ErrorResponseError:
    """
    Attributes:
        code (ErrorCodes):
        message (str):
        detail (Union[Any, None, Unset]):
    """

    code: ErrorCodes
    message: str
    detail: Union[Any, None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code.value

        message = self.message

        detail: Union[Any, None, Unset]
        if isinstance(self.detail, Unset):
            detail = UNSET
        else:
            detail = self.detail

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "code": code,
                "message": message,
            }
        )
        if detail is not UNSET:
            field_dict["detail"] = detail

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = ErrorCodes(d.pop("code"))

        message = d.pop("message")

        def _parse_detail(data: object) -> Union[Any, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, None, Unset], data)

        detail = _parse_detail(d.pop("detail", UNSET))

        error_response_error = cls(
            code=code,
            message=message,
            detail=detail,
        )

        error_response_error.additional_properties = d
        return error_response_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import List, Optional, Dict
import json
from pydantic import BaseModel
from datetime import datetime
from maihem.schemas.agents import AgentType
from maihem.api_client.maihem_client.models.conversation_nested import (
    ConversationNested,
)
from enum import Enum


class TestStatusEnum(str, Enum):
    PAUSED = "paused"
    PENDING = "pending"
    RUNNING = "running"
    COMPLETED = "completed"
    FAILED = "failed"


class TestResultEnum(str, Enum):
    PENDING = "pending"
    PASSED = "passed"
    FAILED = "failed"
    ERRORED = "errored"
    CANCELLED = "cancelled"


class APISchemaLinks(BaseModel):
    test_conversations: Optional[str] = None
    test_result: Optional[str] = None
    test_result_conversations: Optional[str] = None

    class Config:
        exclude_none = True


class Test(BaseModel):
    id: str
    created_at: datetime
    updated_at: datetime
    identifier: str
    name: Optional[str] = None
    initiating_agent: AgentType = AgentType.MAIHEM
    conversation_turns_max: Optional[int] = None
    agent_maihem_behavior_prompt: Optional[str] = None
    metrics_config: Dict[str, int] = {}


class TestRun(BaseModel):
    id: str
    created_at: datetime
    updated_at: datetime
    test_id: str
    agent_target_id: str
    started_at: Optional[datetime] = None
    completed_at: Optional[datetime] = None
    status: TestStatusEnum
    result: TestResultEnum
    links: Optional[APISchemaLinks] = None

    def __str__(self):
        return json.dumps(self.model_dump(), default=str, indent=4)


class TestRunConversations(TestRun):
    conversation_ids: List[str] = []

    def __str__(self):
        return json.dumps(self.model_dump(), default=str, indent=4)


class TestRunResultMetricScore(BaseModel):
    total: int
    passed: int
    failed: int
    errored: int


class ConversationCounts(BaseModel):
    total_conversations: int
    result_passed: int
    result_failed: int
    result_errored: int
    result_pending: int
    result_cancelled: int
    status_completed: int
    status_failed: int
    status_running: int
    status_pending: int
    status_paused: int
    total_score: Optional[float] = None


class TestRunResultMetrics(TestRun):
    conversation_counts: ConversationCounts
    metric_scores: Dict[str, TestRunResultMetricScore] = {}


class TestRunResultConversations(TestRun):
    conversations: List[ConversationNested] = []

    def __str__(self):
        test_run_dict = self.model_dump()
        conversation_jsons = [
            conversation.to_dict() for conversation in self.conversations
        ]
        test_run_dict["conversations"] = conversation_jsons

        return json.dumps(test_run_dict, default=str, indent=4)

    def to_json(self):
        return self.__str__()

    def to_dict(self):
        test_run_dict = self.model_dump()
        conversation_jsons = [
            conversation.to_dict() for conversation in self.conversations
        ]
        test_run_dict["conversations"] = conversation_jsons

        return test_run_dict

    class Config:
        arbitrary_types_allowed = True



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="APISchemaLinks")


@_attrs_define
class APISchemaLinks:
    """
    Attributes:
        test_conversations (Union[None, Unset, str]):
        test_result (Union[None, Unset, str]):
        test_result_conversations (Union[None, Unset, str]):
    """

    test_conversations: Union[None, Unset, str] = UNSET
    test_result: Union[None, Unset, str] = UNSET
    test_result_conversations: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        test_conversations: Union[None, Unset, str]
        if isinstance(self.test_conversations, Unset):
            test_conversations = UNSET
        else:
            test_conversations = self.test_conversations

        test_result: Union[None, Unset, str]
        if isinstance(self.test_result, Unset):
            test_result = UNSET
        else:
            test_result = self.test_result

        test_result_conversations: Union[None, Unset, str]
        if isinstance(self.test_result_conversations, Unset):
            test_result_conversations = UNSET
        else:
            test_result_conversations = self.test_result_conversations

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if test_conversations is not UNSET:
            field_dict["test_conversations"] = test_conversations
        if test_result is not UNSET:
            field_dict["test_result"] = test_result
        if test_result_conversations is not UNSET:
            field_dict["test_result_conversations"] = test_result_conversations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_test_conversations(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        test_conversations = _parse_test_conversations(d.pop("test_conversations", UNSET))

        def _parse_test_result(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        test_result = _parse_test_result(d.pop("test_result", UNSET))

        def _parse_test_result_conversations(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        test_result_conversations = _parse_test_result_conversations(d.pop("test_result_conversations", UNSET))

        api_schema_links = cls(
            test_conversations=test_conversations,
            test_result=test_result,
            test_result_conversations=test_result_conversations,
        )

        api_schema_links.additional_properties = d
        return api_schema_links

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="APISchemaUser")


@_attrs_define
class APISchemaUser:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        idp_user_id (str):
        first_name (str):
        last_name (str):
        email (str):
        image_url (Union[None, Unset, str]):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    idp_user_id: str
    first_name: str
    last_name: str
    email: str
    image_url: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        idp_user_id = self.idp_user_id

        first_name = self.first_name

        last_name = self.last_name

        email = self.email

        image_url: Union[None, Unset, str]
        if isinstance(self.image_url, Unset):
            image_url = UNSET
        else:
            image_url = self.image_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "idp_user_id": idp_user_id,
                "first_name": first_name,
                "last_name": last_name,
                "email": email,
            }
        )
        if image_url is not UNSET:
            field_dict["image_url"] = image_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        idp_user_id = d.pop("idp_user_id")

        first_name = d.pop("first_name")

        last_name = d.pop("last_name")

        email = d.pop("email")

        def _parse_image_url(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        image_url = _parse_image_url(d.pop("image_url", UNSET))

        api_schema_user = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            idp_user_id=idp_user_id,
            first_name=first_name,
            last_name=last_name,
            email=email,
            image_url=image_url,
        )

        api_schema_user.additional_properties = d
        return api_schema_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Tuple, List
import os
import random

from maihem.clients import Maihem
from maihem.utils import extract_text


# Create a Maihem client by providing your API key. 
# Add API key to MAIHEM_API_KEY environment variable.
# os.environ["MAIHEM_API_KEY"] = api_key
maihem_client = Maihem()

# Create (and describe) the target agent you want to test
try:
    target_agent = maihem_client.create_target_agent(
        identifier="tg_agent_pg_essays",
        role="question answering",
        industry="entrepreneurship",
        description="A chatbot that answers questions about entrepreneurship and startups, based in the essays of Paul Graham, the co-founder of Y Combinator.",
    )
except Exception as e:
    # or get target agent if it exists
    target_agent = maihem_client.get_target_agent(identifier="tg_agent_pg_essays")

# Example of chat function
def chat_function(conversation_id: str, agent_maihem_message: str) -> Tuple[str, List[str]]:
    """
    Callable chat function to wrap your target agent to be tested.

    params:
    - conversation_id: unique conversation ID generated by Maihem, use it to keep track of different conversations
    - agent_maihem_msg: message from the Maihem Agent

    return:
    - agent_target_message [str] - message from your Target Agent in response to the Maihem Agent message
    - contexts [List[str]] - list of contexts (required for RAG evaluations), pass empty list if not needed
    """
    
    # The message from your Target Agent. 
    # Connect your agent here to respond to agent_maihem_message. 
    # Keep track of same or new conversations with conversation_id
    agent_target_message = "Hi, how can I help you today?"
    
    # List of contexts for RAG evaluations, pass empty list if not needed
    contexts = ["<Context 1>", "<Context 2>"] 
    
    return agent_target_message, contexts
    
# Set the chat function to the Target Agent, so we can automatically chat with your Target Agent
target_agent.set_chat_function(chat_function)

# For RAG testing, add a list of documents so Maihem can generate questions from them to evaluate your Target Agent
target_agent.add_documents(documents=["./src/test_data/RAG_PG_essays.pdf"])

# Select the Metrics you want to evaluate your agent on, 
# and how many conversations you want to run for each metric.
# You can find the list of supported metrics here: https://docs.maihem.ai/metrics/collection
metrics_config = {
    "qa_cx_helpfulness": 2, # Number of conversations to run for this metric
    "qa_cx_goal_completion": 2, # Number of conversations to run for this metric
    "qa_rag_hallucination": 2, # Number of conversations to run for this metric
    "qa_rag_answer_relevance": 2, # Number of conversations to run for this metric
}

# Create a Test
test = maihem_client.create_test(
    identifier="test_pg_essays_rag",
    metrics_config=metrics_config,
    maihem_agent_behavior_prompt="Ask questions about entrepreneurship and startups based on the essays of Paul Graham. Image you are a young entrepreneur looking for advice, deciding wheter to drop out of college to start a company.",
    conversation_turns_max=5
)

# # Create a Test Run from a Test and a Target Agent
test_run = maihem_client.create_test_run(
    test_identifier="test_pg_essays_rag", # The identifier of the Test you want to run
    target_agent=target_agent, # The Target Agent you want to test
    concurrent_conversations=10, # How many conversations to run concurrently
)

# Get the results of the Test Run (status and how many conversations passed or failed)
results = maihem_client.get_test_run_result(test_run_id=test_run.id)
print(results)

# Get the conversations generated in the Test Run
conversations = maihem_client.get_test_run_result_conversations(test_run_id=test_run.id)
print(conversations)



"""Contains all the data models used in inputs/outputs"""

from .agent_maihem_role import AgentMaihemRole
from .agent_type import AgentType
from .api_schema_agent_target import APISchemaAgentTarget
from .api_schema_agent_target_create_request import APISchemaAgentTargetCreateRequest
from .api_schema_conversation_turn_create_request import APISchemaConversationTurnCreateRequest
from .api_schema_conversation_turn_create_request_document_type_0 import (
    APISchemaConversationTurnCreateRequestDocumentType0,
)
from .api_schema_conversation_turn_create_response import APISchemaConversationTurnCreateResponse
from .api_schema_links import APISchemaLinks
from .api_schema_metric import APISchemaMetric
from .api_schema_org import APISchemaOrg
from .api_schema_test import APISchemaTest
from .api_schema_test_create_request import APISchemaTestCreateRequest
from .api_schema_test_create_request_metrics_config import APISchemaTestCreateRequestMetricsConfig
from .api_schema_test_metrics_config import APISchemaTestMetricsConfig
from .api_schema_test_result_metric_feedback import APISchemaTestResultMetricFeedback
from .api_schema_test_result_metric_feedback_create_request import APISchemaTestResultMetricFeedbackCreateRequest
from .api_schema_test_result_metric_feedback_create_request_feedback import (
    APISchemaTestResultMetricFeedbackCreateRequestFeedback,
)
from .api_schema_test_result_metric_feedback_feedback import APISchemaTestResultMetricFeedbackFeedback
from .api_schema_test_run import APISchemaTestRun
from .api_schema_test_run_conversation_counts import APISchemaTestRunConversationCounts
from .api_schema_test_run_conversations import APISchemaTestRunConversations
from .api_schema_test_run_create_request import APISchemaTestRunCreateRequest
from .api_schema_test_run_result_conversations import APISchemaTestRunResultConversations
from .api_schema_test_run_result_conversations_metric_scores_type_0 import (
    APISchemaTestRunResultConversationsMetricScoresType0,
)
from .api_schema_test_run_result_metric_scores import APISchemaTestRunResultMetricScores
from .api_schema_test_run_result_metrics import APISchemaTestRunResultMetrics
from .api_schema_test_run_result_metrics_metric_scores_type_0 import APISchemaTestRunResultMetricsMetricScoresType0
from .api_schema_user import APISchemaUser
from .api_schema_user_profile import APISchemaUserProfile
from .conversation_nested import ConversationNested
from .conversation_nested_evaluation import ConversationNestedEvaluation
from .conversation_nested_message import ConversationNestedMessage
from .conversation_nested_sentence import ConversationNestedSentence
from .conversation_nested_token_cost import ConversationNestedTokenCost
from .conversation_nested_turn import ConversationNestedTurn
from .error_codes import ErrorCodes
from .error_response import ErrorResponse
from .error_response_error import ErrorResponseError
from .http_validation_error import HTTPValidationError
from .idp_org_create_request import IDPOrgCreateRequest
from .idp_org_user_add_request import IDPOrgUserAddRequest
from .idp_user_create_request import IDPUserCreateRequest
from .idp_user_update_request import IDPUserUpdateRequest
from .org import Org
from .org_base import OrgBase
from .test_result_enum import TestResultEnum
from .test_status_enum import TestStatusEnum
from .validation_error import ValidationError

__all__ = (
    "AgentMaihemRole",
    "AgentType",
    "APISchemaAgentTarget",
    "APISchemaAgentTargetCreateRequest",
    "APISchemaConversationTurnCreateRequest",
    "APISchemaConversationTurnCreateRequestDocumentType0",
    "APISchemaConversationTurnCreateResponse",
    "APISchemaLinks",
    "APISchemaMetric",
    "APISchemaOrg",
    "APISchemaTest",
    "APISchemaTestCreateRequest",
    "APISchemaTestCreateRequestMetricsConfig",
    "APISchemaTestMetricsConfig",
    "APISchemaTestResultMetricFeedback",
    "APISchemaTestResultMetricFeedbackCreateRequest",
    "APISchemaTestResultMetricFeedbackCreateRequestFeedback",
    "APISchemaTestResultMetricFeedbackFeedback",
    "APISchemaTestRun",
    "APISchemaTestRunConversationCounts",
    "APISchemaTestRunConversations",
    "APISchemaTestRunCreateRequest",
    "APISchemaTestRunResultConversations",
    "APISchemaTestRunResultConversationsMetricScoresType0",
    "APISchemaTestRunResultMetrics",
    "APISchemaTestRunResultMetricScores",
    "APISchemaTestRunResultMetricsMetricScoresType0",
    "APISchemaUser",
    "APISchemaUserProfile",
    "ConversationNested",
    "ConversationNestedEvaluation",
    "ConversationNestedMessage",
    "ConversationNestedSentence",
    "ConversationNestedTokenCost",
    "ConversationNestedTurn",
    "ErrorCodes",
    "ErrorResponse",
    "ErrorResponseError",
    "HTTPValidationError",
    "IDPOrgCreateRequest",
    "IDPOrgUserAddRequest",
    "IDPUserCreateRequest",
    "IDPUserUpdateRequest",
    "Org",
    "OrgBase",
    "TestResultEnum",
    "TestStatusEnum",
    "ValidationError",
)



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="IDPOrgUserAddRequest")


@_attrs_define
class IDPOrgUserAddRequest:
    """
    Attributes:
        event_type (str):
        org_id (str):
        role (str):
        user_id (str):
    """

    event_type: str
    org_id: str
    role: str
    user_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        event_type = self.event_type

        org_id = self.org_id

        role = self.role

        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "event_type": event_type,
                "org_id": org_id,
                "role": role,
                "user_id": user_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        event_type = d.pop("event_type")

        org_id = d.pop("org_id")

        role = d.pop("role")

        user_id = d.pop("user_id")

        idp_org_user_add_request = cls(
            event_type=event_type,
            org_id=org_id,
            role=role,
            user_id=user_id,
        )

        idp_org_user_add_request.additional_properties = d
        return idp_org_user_add_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_schema_conversation_turn_create_request_document_type_0 import (
        APISchemaConversationTurnCreateRequestDocumentType0,
    )


T = TypeVar("T", bound="APISchemaConversationTurnCreateRequest")


@_attrs_define
class APISchemaConversationTurnCreateRequest:
    """
    Attributes:
        message (Union[None, Unset, str]):
        contexts (Union[List[str], None, Unset]):
        document (Union['APISchemaConversationTurnCreateRequestDocumentType0', None, Unset]):
    """

    message: Union[None, Unset, str] = UNSET
    contexts: Union[List[str], None, Unset] = UNSET
    document: Union["APISchemaConversationTurnCreateRequestDocumentType0", None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.api_schema_conversation_turn_create_request_document_type_0 import (
            APISchemaConversationTurnCreateRequestDocumentType0,
        )

        message: Union[None, Unset, str]
        if isinstance(self.message, Unset):
            message = UNSET
        else:
            message = self.message

        contexts: Union[List[str], None, Unset]
        if isinstance(self.contexts, Unset):
            contexts = UNSET
        elif isinstance(self.contexts, list):
            contexts = self.contexts

        else:
            contexts = self.contexts

        document: Union[Dict[str, Any], None, Unset]
        if isinstance(self.document, Unset):
            document = UNSET
        elif isinstance(self.document, APISchemaConversationTurnCreateRequestDocumentType0):
            document = self.document.to_dict()
        else:
            document = self.document

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if message is not UNSET:
            field_dict["message"] = message
        if contexts is not UNSET:
            field_dict["contexts"] = contexts
        if document is not UNSET:
            field_dict["document"] = document

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_schema_conversation_turn_create_request_document_type_0 import (
            APISchemaConversationTurnCreateRequestDocumentType0,
        )

        d = src_dict.copy()

        def _parse_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        message = _parse_message(d.pop("message", UNSET))

        def _parse_contexts(data: object) -> Union[List[str], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                contexts_type_0 = cast(List[str], data)

                return contexts_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[str], None, Unset], data)

        contexts = _parse_contexts(d.pop("contexts", UNSET))

        def _parse_document(data: object) -> Union["APISchemaConversationTurnCreateRequestDocumentType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                document_type_0 = APISchemaConversationTurnCreateRequestDocumentType0.from_dict(data)

                return document_type_0
            except:  # noqa: E722
                pass
            return cast(Union["APISchemaConversationTurnCreateRequestDocumentType0", None, Unset], data)

        document = _parse_document(d.pop("document", UNSET))

        api_schema_conversation_turn_create_request = cls(
            message=message,
            contexts=contexts,
            document=document,
        )

        api_schema_conversation_turn_create_request.additional_properties = d
        return api_schema_conversation_turn_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Optional
import json
from maihem.api_client.maihem_client.client import Client as MaihemHTTPClient
from maihem.api_client.maihem_client.types import Response
from maihem.api_client.maihem_client.models.api_schema_agent_target_create_request import (
    APISchemaAgentTargetCreateRequest,
)
from maihem.api_client.maihem_client.models.api_schema_agent_target import (
    APISchemaAgentTarget,
)
from maihem.api_client.maihem_client.models.api_schema_test_create_request import (
    APISchemaTestCreateRequest,
)
from maihem.api_client.maihem_client.models.api_schema_test_run_create_request import (
    APISchemaTestRunCreateRequest,
)
from maihem.api_client.maihem_client.models.api_schema_test import (
    APISchemaTest,
)
from maihem.api_client.maihem_client.models.api_schema_test_run import (
    APISchemaTestRun,
)
from maihem.api_client.maihem_client.models.api_schema_test_run_result_metrics import (
    APISchemaTestRunResultMetrics,
)
from maihem.api_client.maihem_client.models.api_schema_test_run_result_conversations import (
    APISchemaTestRunResultConversations,
)
from maihem.api_client.maihem_client.models.api_schema_test_run_conversations import (
    APISchemaTestRunConversations,
)
from maihem.api_client.maihem_client.api.tests import tests_create_test
from maihem.api_client.maihem_client.api.tests import (
    tests_create_test_run,
    tests_get_tests,
)
from maihem.api_client.maihem_client.models.conversation_nested import (
    ConversationNested,
)
from maihem.api_client.maihem_client.models.api_schema_conversation_turn_create_request import (
    APISchemaConversationTurnCreateRequest,
)
from maihem.api_client.maihem_client.models.api_schema_conversation_turn_create_request_document_type_0 import (
    APISchemaConversationTurnCreateRequestDocumentType0,
)
from maihem.api_client.maihem_client.models.api_schema_conversation_turn_create_response import (
    APISchemaConversationTurnCreateResponse,
)
from maihem.api_client.maihem_client.api.test_runs import (
    test_runs_get_test_run,
    test_runs_get_test_run_conversations,
    test_runs_get_test_run_result_with_conversations,
    test_runs_create_conversation_turn,
    test_runs_get_test_run_result,
)
from maihem.api_client.maihem_client.api.conversations import (
    conversations_get_conversation,
)
from maihem.api_client.maihem_client.api.whoami import whoami_who_am_i
from maihem.api_client.maihem_client.api.agents import agents_create_agent_target
from maihem.api_client.maihem_client.api.agents import (
    agents_get_agent_targets,
)

from maihem.errors import handle_http_errors, ErrorResponse


class MaihemHTTPClientBase:
    def __init__(self, base_url: str, token: str):
        self.base_url = base_url
        self.token = token


class MaihemHTTPClientSync(MaihemHTTPClientBase):
    def whoami(self) -> Dict[str, str]:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response = whoami_who_am_i.sync_detailed(
                client=client, x_api_key=self.token
            )

        if response.status_code != 200:
            handle_http_errors(error_resp=response.parsed)
        return response.parsed

    def create_agent_target(
        self, req: APISchemaAgentTargetCreateRequest
    ) -> APISchemaAgentTarget:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaAgentTarget] = (
                agents_create_agent_target.sync_detailed(
                    client=client,
                    x_api_key=self.token,
                    body=req,
                )
            )

        if response.status_code != 201:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def get_agent_target_by_identifier(
        self, identifier: str
    ) -> APISchemaAgentTarget:

        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[List[APISchemaAgentTarget]] = (
                agents_get_agent_targets.sync_detailed(
                    client=client,
                    x_api_key=self.token,
                    identifier=identifier,
                )
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))

        return response.parsed[0]

    def create_test(self, req: APISchemaTestCreateRequest) -> APISchemaTest:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaTest] = tests_create_test.sync_detailed(
                client=client,
                x_api_key=self.token,
                body=req,
            )

        if response.status_code != 201:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))

        return response.parsed

    def get_test_by_identifier(self, identifier: str) -> APISchemaTest:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[List[APISchemaTest]] = tests_get_tests.sync_detailed(
                client=client,
                x_api_key=self.token,
                identifier=identifier,
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))

        return response.parsed[0]

    def create_test_run(self, test_id: str, agent_target_id: str) -> APISchemaTestRun:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaTestRun] = tests_create_test_run.sync_detailed(
                client=client,
                x_api_key=self.token,
                test_id=test_id,
                body=APISchemaTestRunCreateRequest(agent_target_id=agent_target_id),
            )

        if response.status_code != 201:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def create_conversation_turn(
        self,
        test_run_id: str,
        conversation_id: str,
        target_agent_message: Optional[str] = None,
        contexts: Optional[List[str]] = None,
        document: Optional[Dict[str, str]] = None,
    ) -> APISchemaConversationTurnCreateResponse:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            document_req = None

            if document:
                document_req = (
                    APISchemaConversationTurnCreateRequestDocumentType0.from_dict(
                        document
                    )
                )

            req: APISchemaConversationTurnCreateRequest = (
                APISchemaConversationTurnCreateRequest(
                    message=target_agent_message,
                    contexts=contexts,
                    document=document_req if document_req else None,
                )
            )

            response: Response[APISchemaConversationTurnCreateResponse] = (
                test_runs_create_conversation_turn.sync_detailed(
                    client=client,
                    x_api_key=self.token,
                    test_run_id=test_run_id,
                    conversation_id=conversation_id,
                    body=req,
                )
            )

        if response.status_code != 201:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def get_conversation(self, conversation_id: str) -> ConversationNested:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[ConversationNested] = (
                conversations_get_conversation.sync_detailed(
                    client=client,
                    x_api_key=self.token,
                    conversation_id=conversation_id,
                )
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def get_test_run(self, test_run_id: str) -> APISchemaTestRun:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaTestRun] = test_runs_get_test_run.sync_detailed(
                client=client, x_api_key=self.token, test_run_id=test_run_id
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def get_test_run_conversations(
        self, test_run_id: str
    ) -> APISchemaTestRunConversations:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaTestRunConversations] = (
                test_runs_get_test_run_conversations.sync_detailed(
                    client=client, x_api_key=self.token, test_run_id=test_run_id
                )
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def get_test_run_result(self, test_run_id: str) -> APISchemaTestRunResultMetrics:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaTestRunResultMetrics] = (
                test_runs_get_test_run_result.sync_detailed(
                    client=client, x_api_key=self.token, test_run_id=test_run_id
                )
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed

    def get_test_run_result_conversations(
        self, test_run_id: str
    ) -> APISchemaTestRunResultConversations:
        with MaihemHTTPClient(base_url=self.base_url) as client:
            response: Response[APISchemaTestRunResultConversations] = (
                test_runs_get_test_run_result_with_conversations.sync_detailed(
                    client=client, x_api_key=self.token, test_run_id=test_run_id
                )
            )

        if response.status_code != 200:
            error_dict = json.loads(response.content)
            handle_http_errors(error_resp=ErrorResponse.from_dict(error_dict))
        return response.parsed



import os
from loguru import logger
import sys


class Logger:
    def __init__(self, name: str, level="DEBUG"):
        self.name = name

        # Remove default handler
        logger.remove()

        # Add custom handler
        logger.add(
            sys.stdout,
            colorize=True,
            format=self.format_message,
            level=level,
            filter=self.add_extra_fields,
        )

    def format_message(self, record):
        base_format = "<blue>{time:YYYY-MM-DD HH:mm:ss}</blue> - <level>{level}</level> - {message}"
        if record["extra"]["request_id"]:
            base_format += " - <dim>request_id:{extra[request_id]}</dim>"
        base_format += "\n"
        return base_format

    def add_extra_fields(self, record):
        record["extra"]["request_id"] = ""
        return True

    def debug(self, msg: str, *args, **kwargs):
        logger.debug(msg, *args, **kwargs)

    def info(self, msg: str, *args, **kwargs):
        logger.info(msg, *args, **kwargs)

    def warning(self, msg: str, *args, **kwargs):
        logger.warning(msg, *args, **kwargs)

    def error(self, msg: str, *args, **kwargs):
        logger.error(msg, *args, **kwargs)

    def critical(self, msg: str, *args, **kwargs):
        logger.critical(msg, *args, **kwargs)

    def exception(self, msg: str, *args, **kwargs):
        logger.exception(msg, *args, **kwargs)


logger_instance = Logger(name="maihem-python-sdk")


def get_logger():
    return logger_instance



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="APISchemaMetric")


@_attrs_define
class APISchemaMetric:
    """
    Attributes:
        id (str):
        metric_group (str):
        metric_group_label (str):
        metric_sub_group (str):
        metric_sub_group_label (str):
        metric (str):
        metric_label (str):
    """

    id: str
    metric_group: str
    metric_group_label: str
    metric_sub_group: str
    metric_sub_group_label: str
    metric: str
    metric_label: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        metric_group = self.metric_group

        metric_group_label = self.metric_group_label

        metric_sub_group = self.metric_sub_group

        metric_sub_group_label = self.metric_sub_group_label

        metric = self.metric

        metric_label = self.metric_label

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "metric_group": metric_group,
                "metric_group_label": metric_group_label,
                "metric_sub_group": metric_sub_group,
                "metric_sub_group_label": metric_sub_group_label,
                "metric": metric,
                "metric_label": metric_label,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        metric_group = d.pop("metric_group")

        metric_group_label = d.pop("metric_group_label")

        metric_sub_group = d.pop("metric_sub_group")

        metric_sub_group_label = d.pop("metric_sub_group_label")

        metric = d.pop("metric")

        metric_label = d.pop("metric_label")

        api_schema_metric = cls(
            id=id,
            metric_group=metric_group,
            metric_group_label=metric_group_label,
            metric_sub_group=metric_sub_group,
            metric_sub_group_label=metric_sub_group_label,
            metric=metric,
            metric_label=metric_label,
        )

        api_schema_metric.additional_properties = d
        return api_schema_metric

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from datetime import datetime
from pydantic import BaseModel
from typing import Callable, Optional, Tuple, List, Dict
import maihem.errors as errors
from enum import Enum
from maihem.logger import get_logger
from pydantic_extra_types.language_code import LanguageAlpha2
import os
from maihem.utils import extract_text


class TargetAgent(BaseModel):
    id: str
    created_at: datetime
    updated_at: datetime
    identifier: str
    name: Optional[str] = None
    role: str
    description: Optional[str] = None
    industry: Optional[str] = None
    language: Optional[LanguageAlpha2] = "en"

    _chat_function: Optional[Callable] = None
    document_paths: List[str] = []

    def set_chat_function(self, chat_function: Callable) -> None:
        logger = get_logger()
        is_test_success = self.test_chat_function(chat_function)

        if is_test_success:
            self._chat_function = chat_function
            logger.info("Chat function tested and added successfully!")

    def test_chat_function(self, chat_function: Callable) -> bool:
        logger = get_logger()
        test_message = "Hi, it's the Maihem agent. Ready for testing?"
        logger.info("Testing chat function...")
        logger.info(f"Sending chatbot message: {test_message}")
        try:
            message, contexts = chat_function(
                str(datetime.now()),
                test_message,
            )
            assert isinstance(message, str), "Response message must be a string"
            assert isinstance(contexts, list), "Contexts must be a list"
            for context in contexts:
                assert isinstance(
                    context, str
                ), "Each context in the list must be a string"

            return True
        except Exception as e:
            errors.raise_chat_function_error(
                f"Error testing target agent chat function: {e}"
            )

    def add_documents(self, documents: List[str]) -> None:
        logger = get_logger()
        for doc_path in documents:
            if not os.path.exists(doc_path):
                logger.warning(f"Document not found: {doc_path}")
                continue

            if not os.path.isfile(doc_path):
                logger.warning(f"Path is not a file: {doc_path}")
                continue

            file_size = os.path.getsize(doc_path)
            if file_size == 0:
                logger.warning(f"File is empty: {doc_path}")
                continue

            if file_size > 10 * 1024 * 1024:  # 10 MB limit
                logger.warning(f"File is too large (>10MB): {doc_path}")
                continue

            try:
                self.document_paths.append(doc_path)
                logger.info(f"Added document: {doc_path}")
            except Exception as e:
                logger.error(f"Error processing document {doc_path}: {str(e)}")

    def _send_message(
        self, conversation_id: str, message: Optional[str] = ""
    ) -> Tuple[str, List[str]]:
        if not self._chat_function:
            errors.raise_chat_function_error("Target agent chat function not set")
        response, contexts = self._chat_function(conversation_id, message)
        return response, contexts


class AgentType(str, Enum):
    MAIHEM = "maihem"
    TARGET = "target"



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.api_schema_org import APISchemaOrg
    from ..models.api_schema_user import APISchemaUser


T = TypeVar("T", bound="APISchemaUserProfile")


@_attrs_define
class APISchemaUserProfile:
    """
    Attributes:
        user (APISchemaUser):
        org (APISchemaOrg):
    """

    user: "APISchemaUser"
    org: "APISchemaOrg"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user = self.user.to_dict()

        org = self.org.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
                "org": org,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_schema_org import APISchemaOrg
        from ..models.api_schema_user import APISchemaUser

        d = src_dict.copy()
        user = APISchemaUser.from_dict(d.pop("user"))

        org = APISchemaOrg.from_dict(d.pop("org"))

        api_schema_user_profile = cls(
            user=user,
            org=org,
        )

        api_schema_user_profile.additional_properties = d
        return api_schema_user_profile

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.api_schema_test_result_metric_feedback_create_request_feedback import (
    APISchemaTestResultMetricFeedbackCreateRequestFeedback,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="APISchemaTestResultMetricFeedbackCreateRequest")


@_attrs_define
class APISchemaTestResultMetricFeedbackCreateRequest:
    """
    Attributes:
        feedback (Union[Unset, APISchemaTestResultMetricFeedbackCreateRequestFeedback]):  Default:
            APISchemaTestResultMetricFeedbackCreateRequestFeedback.POSITIVE.
        comments (Union[None, Unset, str]):
    """

    feedback: Union[Unset, APISchemaTestResultMetricFeedbackCreateRequestFeedback] = (
        APISchemaTestResultMetricFeedbackCreateRequestFeedback.POSITIVE
    )
    comments: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        feedback: Union[Unset, str] = UNSET
        if not isinstance(self.feedback, Unset):
            feedback = self.feedback.value

        comments: Union[None, Unset, str]
        if isinstance(self.comments, Unset):
            comments = UNSET
        else:
            comments = self.comments

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if feedback is not UNSET:
            field_dict["feedback"] = feedback
        if comments is not UNSET:
            field_dict["comments"] = comments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _feedback = d.pop("feedback", UNSET)
        feedback: Union[Unset, APISchemaTestResultMetricFeedbackCreateRequestFeedback]
        if isinstance(_feedback, Unset):
            feedback = UNSET
        else:
            feedback = APISchemaTestResultMetricFeedbackCreateRequestFeedback(_feedback)

        def _parse_comments(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        comments = _parse_comments(d.pop("comments", UNSET))

        api_schema_test_result_metric_feedback_create_request = cls(
            feedback=feedback,
            comments=comments,
        )

        api_schema_test_result_metric_feedback_create_request.additional_properties = d
        return api_schema_test_result_metric_feedback_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.agent_type import AgentType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_schema_test_create_request_metrics_config import APISchemaTestCreateRequestMetricsConfig


T = TypeVar("T", bound="APISchemaTestCreateRequest")


@_attrs_define
class APISchemaTestCreateRequest:
    """
    Attributes:
        identifier (str):
        metrics_config (APISchemaTestCreateRequestMetricsConfig):
        name (Union[None, Unset, str]):
        initiating_agent (Union[Unset, AgentType]):  Default: AgentType.MAIHEM.
        conversation_turns_max (Union[None, Unset, int]):
        agent_maihem_behavior_prompt (Union[None, Unset, str]):
    """

    identifier: str
    metrics_config: "APISchemaTestCreateRequestMetricsConfig"
    name: Union[None, Unset, str] = UNSET
    initiating_agent: Union[Unset, AgentType] = AgentType.MAIHEM
    conversation_turns_max: Union[None, Unset, int] = UNSET
    agent_maihem_behavior_prompt: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        identifier = self.identifier

        metrics_config = self.metrics_config.to_dict()

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        initiating_agent: Union[Unset, str] = UNSET
        if not isinstance(self.initiating_agent, Unset):
            initiating_agent = self.initiating_agent.value

        conversation_turns_max: Union[None, Unset, int]
        if isinstance(self.conversation_turns_max, Unset):
            conversation_turns_max = UNSET
        else:
            conversation_turns_max = self.conversation_turns_max

        agent_maihem_behavior_prompt: Union[None, Unset, str]
        if isinstance(self.agent_maihem_behavior_prompt, Unset):
            agent_maihem_behavior_prompt = UNSET
        else:
            agent_maihem_behavior_prompt = self.agent_maihem_behavior_prompt

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "identifier": identifier,
                "metrics_config": metrics_config,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if initiating_agent is not UNSET:
            field_dict["initiating_agent"] = initiating_agent
        if conversation_turns_max is not UNSET:
            field_dict["conversation_turns_max"] = conversation_turns_max
        if agent_maihem_behavior_prompt is not UNSET:
            field_dict["agent_maihem_behavior_prompt"] = agent_maihem_behavior_prompt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_schema_test_create_request_metrics_config import APISchemaTestCreateRequestMetricsConfig

        d = src_dict.copy()
        identifier = d.pop("identifier")

        metrics_config = APISchemaTestCreateRequestMetricsConfig.from_dict(d.pop("metrics_config"))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        _initiating_agent = d.pop("initiating_agent", UNSET)
        initiating_agent: Union[Unset, AgentType]
        if isinstance(_initiating_agent, Unset):
            initiating_agent = UNSET
        else:
            initiating_agent = AgentType(_initiating_agent)

        def _parse_conversation_turns_max(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        conversation_turns_max = _parse_conversation_turns_max(d.pop("conversation_turns_max", UNSET))

        def _parse_agent_maihem_behavior_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        agent_maihem_behavior_prompt = _parse_agent_maihem_behavior_prompt(d.pop("agent_maihem_behavior_prompt", UNSET))

        api_schema_test_create_request = cls(
            identifier=identifier,
            metrics_config=metrics_config,
            name=name,
            initiating_agent=initiating_agent,
            conversation_turns_max=conversation_turns_max,
            agent_maihem_behavior_prompt=agent_maihem_behavior_prompt,
        )

        api_schema_test_create_request.additional_properties = d
        return api_schema_test_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.api_schema_conversation_turn_create_request import APISchemaConversationTurnCreateRequest
from ...models.api_schema_conversation_turn_create_response import APISchemaConversationTurnCreateResponse
from ...models.error_response import ErrorResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    test_run_id: str,
    conversation_id: str,
    *,
    body: APISchemaConversationTurnCreateRequest,
    x_api_key: Union[None, Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    if not isinstance(x_api_key, Unset):
        headers["x-api-key"] = x_api_key

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": f"/test-runs/{test_run_id}/conversations/{conversation_id}/turns",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = APISchemaConversationTurnCreateResponse.from_dict(response.json())

        return response_201
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.CONFLICT:
        response_409 = ErrorResponse.from_dict(response.json())

        return response_409
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = ErrorResponse.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ErrorResponse.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.GATEWAY_TIMEOUT:
        response_504 = ErrorResponse.from_dict(response.json())

        return response_504
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    test_run_id: str,
    conversation_id: str,
    *,
    client: AuthenticatedClient,
    body: APISchemaConversationTurnCreateRequest,
    x_api_key: Union[None, Unset, str] = UNSET,
) -> Response[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]:
    """Create a conversation turn

     Create a new turn in a specified conversation

    Args:
        test_run_id (str):
        conversation_id (str):
        x_api_key (Union[None, Unset, str]):
        body (APISchemaConversationTurnCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]
    """

    kwargs = _get_kwargs(
        test_run_id=test_run_id,
        conversation_id=conversation_id,
        body=body,
        x_api_key=x_api_key,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    test_run_id: str,
    conversation_id: str,
    *,
    client: AuthenticatedClient,
    body: APISchemaConversationTurnCreateRequest,
    x_api_key: Union[None, Unset, str] = UNSET,
) -> Optional[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]:
    """Create a conversation turn

     Create a new turn in a specified conversation

    Args:
        test_run_id (str):
        conversation_id (str):
        x_api_key (Union[None, Unset, str]):
        body (APISchemaConversationTurnCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[APISchemaConversationTurnCreateResponse, ErrorResponse]
    """

    return sync_detailed(
        test_run_id=test_run_id,
        conversation_id=conversation_id,
        client=client,
        body=body,
        x_api_key=x_api_key,
    ).parsed


async def asyncio_detailed(
    test_run_id: str,
    conversation_id: str,
    *,
    client: AuthenticatedClient,
    body: APISchemaConversationTurnCreateRequest,
    x_api_key: Union[None, Unset, str] = UNSET,
) -> Response[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]:
    """Create a conversation turn

     Create a new turn in a specified conversation

    Args:
        test_run_id (str):
        conversation_id (str):
        x_api_key (Union[None, Unset, str]):
        body (APISchemaConversationTurnCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]
    """

    kwargs = _get_kwargs(
        test_run_id=test_run_id,
        conversation_id=conversation_id,
        body=body,
        x_api_key=x_api_key,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    test_run_id: str,
    conversation_id: str,
    *,
    client: AuthenticatedClient,
    body: APISchemaConversationTurnCreateRequest,
    x_api_key: Union[None, Unset, str] = UNSET,
) -> Optional[Union[APISchemaConversationTurnCreateResponse, ErrorResponse]]:
    """Create a conversation turn

     Create a new turn in a specified conversation

    Args:
        test_run_id (str):
        conversation_id (str):
        x_api_key (Union[None, Unset, str]):
        body (APISchemaConversationTurnCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[APISchemaConversationTurnCreateResponse, ErrorResponse]
    """

    return (
        await asyncio_detailed(
            test_run_id=test_run_id,
            conversation_id=conversation_id,
            client=client,
            body=body,
            x_api_key=x_api_key,
        )
    ).parsed



from enum import Enum


class ErrorCodes(str, Enum):
    ERR_AUTHENTICATION_FAILED = "err_authentication_failed"
    ERR_DATA_INTEGRITY = "err_data_integrity"
    ERR_INTERNAL_SERVER = "err_internal_server"
    ERR_NOT_FOUND = "err_not_found"
    ERR_REQUEST_TIMEOUT = "err_request_timeout"
    ERR_REQUEST_VALIDATION = "err_request_validation"
    ERR_SCHEMA_VALIDATION = "err_schema_validation"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.test_result_enum import TestResultEnum
from ..models.test_status_enum import TestStatusEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_schema_links import APISchemaLinks


T = TypeVar("T", bound="APISchemaTestRunConversations")


@_attrs_define
class APISchemaTestRunConversations:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        test_id (str):
        agent_target_id (str):
        status (TestStatusEnum):
        result (TestResultEnum):
        started_at (Union[None, Unset, datetime.datetime]):
        completed_at (Union[None, Unset, datetime.datetime]):
        result_score (Union[None, Unset, float]):
        links (Union['APISchemaLinks', None, Unset]):
        conversation_ids (Union[Unset, List[str]]):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    test_id: str
    agent_target_id: str
    status: TestStatusEnum
    result: TestResultEnum
    started_at: Union[None, Unset, datetime.datetime] = UNSET
    completed_at: Union[None, Unset, datetime.datetime] = UNSET
    result_score: Union[None, Unset, float] = UNSET
    links: Union["APISchemaLinks", None, Unset] = UNSET
    conversation_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.api_schema_links import APISchemaLinks

        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        test_id = self.test_id

        agent_target_id = self.agent_target_id

        status = self.status.value

        result = self.result.value

        started_at: Union[None, Unset, str]
        if isinstance(self.started_at, Unset):
            started_at = UNSET
        elif isinstance(self.started_at, datetime.datetime):
            started_at = self.started_at.isoformat()
        else:
            started_at = self.started_at

        completed_at: Union[None, Unset, str]
        if isinstance(self.completed_at, Unset):
            completed_at = UNSET
        elif isinstance(self.completed_at, datetime.datetime):
            completed_at = self.completed_at.isoformat()
        else:
            completed_at = self.completed_at

        result_score: Union[None, Unset, float]
        if isinstance(self.result_score, Unset):
            result_score = UNSET
        else:
            result_score = self.result_score

        links: Union[Dict[str, Any], None, Unset]
        if isinstance(self.links, Unset):
            links = UNSET
        elif isinstance(self.links, APISchemaLinks):
            links = self.links.to_dict()
        else:
            links = self.links

        conversation_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.conversation_ids, Unset):
            conversation_ids = self.conversation_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "test_id": test_id,
                "agent_target_id": agent_target_id,
                "status": status,
                "result": result,
            }
        )
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if result_score is not UNSET:
            field_dict["result_score"] = result_score
        if links is not UNSET:
            field_dict["links"] = links
        if conversation_ids is not UNSET:
            field_dict["conversation_ids"] = conversation_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_schema_links import APISchemaLinks

        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        test_id = d.pop("test_id")

        agent_target_id = d.pop("agent_target_id")

        status = TestStatusEnum(d.pop("status"))

        result = TestResultEnum(d.pop("result"))

        def _parse_started_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                started_at_type_0 = isoparse(data)

                return started_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        started_at = _parse_started_at(d.pop("started_at", UNSET))

        def _parse_completed_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                completed_at_type_0 = isoparse(data)

                return completed_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        completed_at = _parse_completed_at(d.pop("completed_at", UNSET))

        def _parse_result_score(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        result_score = _parse_result_score(d.pop("result_score", UNSET))

        def _parse_links(data: object) -> Union["APISchemaLinks", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                links_type_0 = APISchemaLinks.from_dict(data)

                return links_type_0
            except:  # noqa: E722
                pass
            return cast(Union["APISchemaLinks", None, Unset], data)

        links = _parse_links(d.pop("links", UNSET))

        conversation_ids = cast(List[str], d.pop("conversation_ids", UNSET))

        api_schema_test_run_conversations = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            test_id=test_id,
            agent_target_id=agent_target_id,
            status=status,
            result=result,
            started_at=started_at,
            completed_at=completed_at,
            result_score=result_score,
            links=links,
            conversation_ids=conversation_ids,
        )

        api_schema_test_run_conversations.additional_properties = d
        return api_schema_test_run_conversations

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.agent_maihem_role import AgentMaihemRole
from ..models.agent_type import AgentType
from ..models.test_result_enum import TestResultEnum
from ..models.test_status_enum import TestStatusEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_schema_test_metrics_config import APISchemaTestMetricsConfig


T = TypeVar("T", bound="APISchemaTest")


@_attrs_define
class APISchemaTest:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        identifier (str):
        initiating_agent (AgentType):
        metrics_config (APISchemaTestMetricsConfig):
        name (Union[None, Unset, str]):
        conversation_turns_max (Union[None, Unset, int]):
        agent_maihem_behavior_prompt (Union[None, Unset, str]):
        agent_maihem_roles (Union[Unset, List[AgentMaihemRole]]):
        last_test_run_id (Union[None, Unset, str]):
        last_test_run_at (Union[None, Unset, datetime.datetime]):
        last_test_run_status (Union[None, TestStatusEnum, Unset]):
        last_test_run_result (Union[None, TestResultEnum, Unset]):
        last_test_run_result_score (Union[None, Unset, float]):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    identifier: str
    initiating_agent: AgentType
    metrics_config: "APISchemaTestMetricsConfig"
    name: Union[None, Unset, str] = UNSET
    conversation_turns_max: Union[None, Unset, int] = UNSET
    agent_maihem_behavior_prompt: Union[None, Unset, str] = UNSET
    agent_maihem_roles: Union[Unset, List[AgentMaihemRole]] = UNSET
    last_test_run_id: Union[None, Unset, str] = UNSET
    last_test_run_at: Union[None, Unset, datetime.datetime] = UNSET
    last_test_run_status: Union[None, TestStatusEnum, Unset] = UNSET
    last_test_run_result: Union[None, TestResultEnum, Unset] = UNSET
    last_test_run_result_score: Union[None, Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        identifier = self.identifier

        initiating_agent = self.initiating_agent.value

        metrics_config = self.metrics_config.to_dict()

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        conversation_turns_max: Union[None, Unset, int]
        if isinstance(self.conversation_turns_max, Unset):
            conversation_turns_max = UNSET
        else:
            conversation_turns_max = self.conversation_turns_max

        agent_maihem_behavior_prompt: Union[None, Unset, str]
        if isinstance(self.agent_maihem_behavior_prompt, Unset):
            agent_maihem_behavior_prompt = UNSET
        else:
            agent_maihem_behavior_prompt = self.agent_maihem_behavior_prompt

        agent_maihem_roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.agent_maihem_roles, Unset):
            agent_maihem_roles = []
            for agent_maihem_roles_item_data in self.agent_maihem_roles:
                agent_maihem_roles_item = agent_maihem_roles_item_data.value
                agent_maihem_roles.append(agent_maihem_roles_item)

        last_test_run_id: Union[None, Unset, str]
        if isinstance(self.last_test_run_id, Unset):
            last_test_run_id = UNSET
        else:
            last_test_run_id = self.last_test_run_id

        last_test_run_at: Union[None, Unset, str]
        if isinstance(self.last_test_run_at, Unset):
            last_test_run_at = UNSET
        elif isinstance(self.last_test_run_at, datetime.datetime):
            last_test_run_at = self.last_test_run_at.isoformat()
        else:
            last_test_run_at = self.last_test_run_at

        last_test_run_status: Union[None, Unset, str]
        if isinstance(self.last_test_run_status, Unset):
            last_test_run_status = UNSET
        elif isinstance(self.last_test_run_status, TestStatusEnum):
            last_test_run_status = self.last_test_run_status.value
        else:
            last_test_run_status = self.last_test_run_status

        last_test_run_result: Union[None, Unset, str]
        if isinstance(self.last_test_run_result, Unset):
            last_test_run_result = UNSET
        elif isinstance(self.last_test_run_result, TestResultEnum):
            last_test_run_result = self.last_test_run_result.value
        else:
            last_test_run_result = self.last_test_run_result

        last_test_run_result_score: Union[None, Unset, float]
        if isinstance(self.last_test_run_result_score, Unset):
            last_test_run_result_score = UNSET
        else:
            last_test_run_result_score = self.last_test_run_result_score

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "identifier": identifier,
                "initiating_agent": initiating_agent,
                "metrics_config": metrics_config,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if conversation_turns_max is not UNSET:
            field_dict["conversation_turns_max"] = conversation_turns_max
        if agent_maihem_behavior_prompt is not UNSET:
            field_dict["agent_maihem_behavior_prompt"] = agent_maihem_behavior_prompt
        if agent_maihem_roles is not UNSET:
            field_dict["agent_maihem_roles"] = agent_maihem_roles
        if last_test_run_id is not UNSET:
            field_dict["last_test_run_id"] = last_test_run_id
        if last_test_run_at is not UNSET:
            field_dict["last_test_run_at"] = last_test_run_at
        if last_test_run_status is not UNSET:
            field_dict["last_test_run_status"] = last_test_run_status
        if last_test_run_result is not UNSET:
            field_dict["last_test_run_result"] = last_test_run_result
        if last_test_run_result_score is not UNSET:
            field_dict["last_test_run_result_score"] = last_test_run_result_score

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_schema_test_metrics_config import APISchemaTestMetricsConfig

        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        identifier = d.pop("identifier")

        initiating_agent = AgentType(d.pop("initiating_agent"))

        metrics_config = APISchemaTestMetricsConfig.from_dict(d.pop("metrics_config"))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_conversation_turns_max(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        conversation_turns_max = _parse_conversation_turns_max(d.pop("conversation_turns_max", UNSET))

        def _parse_agent_maihem_behavior_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        agent_maihem_behavior_prompt = _parse_agent_maihem_behavior_prompt(d.pop("agent_maihem_behavior_prompt", UNSET))

        agent_maihem_roles = []
        _agent_maihem_roles = d.pop("agent_maihem_roles", UNSET)
        for agent_maihem_roles_item_data in _agent_maihem_roles or []:
            agent_maihem_roles_item = AgentMaihemRole(agent_maihem_roles_item_data)

            agent_maihem_roles.append(agent_maihem_roles_item)

        def _parse_last_test_run_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_test_run_id = _parse_last_test_run_id(d.pop("last_test_run_id", UNSET))

        def _parse_last_test_run_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_test_run_at_type_0 = isoparse(data)

                return last_test_run_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        last_test_run_at = _parse_last_test_run_at(d.pop("last_test_run_at", UNSET))

        def _parse_last_test_run_status(data: object) -> Union[None, TestStatusEnum, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_test_run_status_type_0 = TestStatusEnum(data)

                return last_test_run_status_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TestStatusEnum, Unset], data)

        last_test_run_status = _parse_last_test_run_status(d.pop("last_test_run_status", UNSET))

        def _parse_last_test_run_result(data: object) -> Union[None, TestResultEnum, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_test_run_result_type_0 = TestResultEnum(data)

                return last_test_run_result_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TestResultEnum, Unset], data)

        last_test_run_result = _parse_last_test_run_result(d.pop("last_test_run_result", UNSET))

        def _parse_last_test_run_result_score(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        last_test_run_result_score = _parse_last_test_run_result_score(d.pop("last_test_run_result_score", UNSET))

        api_schema_test = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            identifier=identifier,
            initiating_agent=initiating_agent,
            metrics_config=metrics_config,
            name=name,
            conversation_turns_max=conversation_turns_max,
            agent_maihem_behavior_prompt=agent_maihem_behavior_prompt,
            agent_maihem_roles=agent_maihem_roles,
            last_test_run_id=last_test_run_id,
            last_test_run_at=last_test_run_at,
            last_test_run_status=last_test_run_status,
            last_test_run_result=last_test_run_result,
            last_test_run_result_score=last_test_run_result_score,
        )

        api_schema_test.additional_properties = d
        return api_schema_test

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IDPUserCreateRequest")


@_attrs_define
class IDPUserCreateRequest:
    """
    Attributes:
        email (str):
        email_confirmed (bool):
        event_type (str):
        first_name (str):
        last_name (str):
        user_id (str):
        picture_url (Union[None, Unset, str]):
        username (Union[None, Unset, str]):
    """

    email: str
    email_confirmed: bool
    event_type: str
    first_name: str
    last_name: str
    user_id: str
    picture_url: Union[None, Unset, str] = UNSET
    username: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email = self.email

        email_confirmed = self.email_confirmed

        event_type = self.event_type

        first_name = self.first_name

        last_name = self.last_name

        user_id = self.user_id

        picture_url: Union[None, Unset, str]
        if isinstance(self.picture_url, Unset):
            picture_url = UNSET
        else:
            picture_url = self.picture_url

        username: Union[None, Unset, str]
        if isinstance(self.username, Unset):
            username = UNSET
        else:
            username = self.username

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "email": email,
                "email_confirmed": email_confirmed,
                "event_type": event_type,
                "first_name": first_name,
                "last_name": last_name,
                "user_id": user_id,
            }
        )
        if picture_url is not UNSET:
            field_dict["picture_url"] = picture_url
        if username is not UNSET:
            field_dict["username"] = username

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email")

        email_confirmed = d.pop("email_confirmed")

        event_type = d.pop("event_type")

        first_name = d.pop("first_name")

        last_name = d.pop("last_name")

        user_id = d.pop("user_id")

        def _parse_picture_url(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        picture_url = _parse_picture_url(d.pop("picture_url", UNSET))

        def _parse_username(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        username = _parse_username(d.pop("username", UNSET))

        idp_user_create_request = cls(
            email=email,
            email_confirmed=email_confirmed,
            event_type=event_type,
            first_name=first_name,
            last_name=last_name,
            user_id=user_id,
            picture_url=picture_url,
            username=username,
        )

        idp_user_create_request.additional_properties = d
        return idp_user_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.test_result_enum import TestResultEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.conversation_nested_token_cost import ConversationNestedTokenCost


T = TypeVar("T", bound="ConversationNestedEvaluation")


@_attrs_define
class ConversationNestedEvaluation:
    """
    Attributes:
        id (str):
        result (TestResultEnum):
        metric_slug (str):
        score (Union[None, Unset, float]):
        confidence (Union[None, Unset, float]):
        explanation (Union[None, Unset, str]):
        classification (Union[None, Unset, str]):
        token_cost (Union['ConversationNestedTokenCost', None, Unset]):
    """

    id: str
    result: TestResultEnum
    metric_slug: str
    score: Union[None, Unset, float] = UNSET
    confidence: Union[None, Unset, float] = UNSET
    explanation: Union[None, Unset, str] = UNSET
    classification: Union[None, Unset, str] = UNSET
    token_cost: Union["ConversationNestedTokenCost", None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.conversation_nested_token_cost import ConversationNestedTokenCost

        id = self.id

        result = self.result.value

        metric_slug = self.metric_slug

        score: Union[None, Unset, float]
        if isinstance(self.score, Unset):
            score = UNSET
        else:
            score = self.score

        confidence: Union[None, Unset, float]
        if isinstance(self.confidence, Unset):
            confidence = UNSET
        else:
            confidence = self.confidence

        explanation: Union[None, Unset, str]
        if isinstance(self.explanation, Unset):
            explanation = UNSET
        else:
            explanation = self.explanation

        classification: Union[None, Unset, str]
        if isinstance(self.classification, Unset):
            classification = UNSET
        else:
            classification = self.classification

        token_cost: Union[Dict[str, Any], None, Unset]
        if isinstance(self.token_cost, Unset):
            token_cost = UNSET
        elif isinstance(self.token_cost, ConversationNestedTokenCost):
            token_cost = self.token_cost.to_dict()
        else:
            token_cost = self.token_cost

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "result": result,
                "metric_slug": metric_slug,
            }
        )
        if score is not UNSET:
            field_dict["score"] = score
        if confidence is not UNSET:
            field_dict["confidence"] = confidence
        if explanation is not UNSET:
            field_dict["explanation"] = explanation
        if classification is not UNSET:
            field_dict["classification"] = classification
        if token_cost is not UNSET:
            field_dict["token_cost"] = token_cost

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.conversation_nested_token_cost import ConversationNestedTokenCost

        d = src_dict.copy()
        id = d.pop("id")

        result = TestResultEnum(d.pop("result"))

        metric_slug = d.pop("metric_slug")

        def _parse_score(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        score = _parse_score(d.pop("score", UNSET))

        def _parse_confidence(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        confidence = _parse_confidence(d.pop("confidence", UNSET))

        def _parse_explanation(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        explanation = _parse_explanation(d.pop("explanation", UNSET))

        def _parse_classification(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        classification = _parse_classification(d.pop("classification", UNSET))

        def _parse_token_cost(data: object) -> Union["ConversationNestedTokenCost", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                token_cost_type_0 = ConversationNestedTokenCost.from_dict(data)

                return token_cost_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ConversationNestedTokenCost", None, Unset], data)

        token_cost = _parse_token_cost(d.pop("token_cost", UNSET))

        conversation_nested_evaluation = cls(
            id=id,
            result=result,
            metric_slug=metric_slug,
            score=score,
            confidence=confidence,
            explanation=explanation,
            classification=classification,
            token_cost=token_cost,
        )

        conversation_nested_evaluation.additional_properties = d
        return conversation_nested_evaluation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class APISchemaTestResultMetricFeedbackCreateRequestFeedback(str, Enum):
    NEGATIVE = "negative"
    POSITIVE = "positive"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum
from maihem.api_client.maihem_client.models.error_response import (
    ErrorResponse,
)
from maihem.api_client.maihem_client.models.error_response_error import (
    ErrorResponseError,
)
from maihem.api_client.maihem_client.models.error_codes import (
    ErrorCodes as ErrorCodesAPI,
)
from pydantic_core import ValidationError
from maihem.logger import get_logger


class ErrorCodes(str, Enum):
    ERR_CHAT_FUNCTION = "err_chat_function"
    ERR_INTERNAL_SERVER = ErrorCodesAPI.ERR_INTERNAL_SERVER
    ERR_NOT_FOUND = ErrorCodesAPI.ERR_NOT_FOUND
    ERR_REQUEST_VALIDATION = ErrorCodesAPI.ERR_REQUEST_VALIDATION
    ERR_SCHEMA_VALIDATION = ErrorCodesAPI.ERR_SCHEMA_VALIDATION
    ERR_DATA_INTEGRITY = ErrorCodesAPI.ERR_DATA_INTEGRITY
    ERR_AUTHENTICATION_FAILED = ErrorCodesAPI.ERR_AUTHENTICATION_FAILED


class ErrorBase(Exception):
    message: str

    def __init__(self, error_resp: ErrorResponse) -> None:
        self.message = error_resp.error.message

    def __str__(self) -> str:
        return self.message


class AuthenticationFailedError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


class InternalServerError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


class NotFoundError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


class RequestValidationError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


class SchemaValidationError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


class DataIntegrityError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


class ChatFunctionError(ErrorBase):
    def __init__(self, error_resp: ErrorResponse) -> None:
        super().__init__(error_resp)


def handle_http_errors(error_resp: ErrorResponse):
    if not error_resp:
        return

    if error_resp.error.code == ErrorCodes.ERR_INTERNAL_SERVER:
        raise InternalServerError(error_resp)
    elif error_resp.error.code == ErrorCodes.ERR_NOT_FOUND:
        raise NotFoundError(error_resp)
    elif error_resp.error.code == ErrorCodes.ERR_REQUEST_VALIDATION:
        raise RequestValidationError(error_resp)
    elif error_resp.error.code == ErrorCodes.ERR_SCHEMA_VALIDATION:
        raise SchemaValidationError(error_resp)
    elif error_resp.error.code == ErrorCodes.ERR_DATA_INTEGRITY:
        raise DataIntegrityError(error_resp)
    elif error_resp.error.code == ErrorCodes.ERR_AUTHENTICATION_FAILED:
        raise AuthenticationFailedError(error_resp)
    else:
        raise ErrorBase(error_resp)


def handle_base_error(exception: ErrorBase):
    get_logger().error(exception.message)
    raise exception


def handle_schema_validation_error(exception: ValidationError):
    raise SchemaValidationError(
        ErrorResponse(
            error=ErrorResponseError(
                code=ErrorCodes.ERR_SCHEMA_VALIDATION,
                message=str(exception.errors()),
                detail=None,
            )
        )
    ) from exception


def raise_request_validation_error(message: str):
    raise RequestValidationError(
        ErrorResponse(
            error=ErrorResponseError(
                code=ErrorCodes.ERR_REQUEST_VALIDATION, message=message, detail=None
            ),
            request_id=None,
        )
    )


def raise_not_found_error(message: str):
    raise NotFoundError(
        ErrorResponse(
            error=ErrorResponseError(
                code=ErrorCodes.ERR_NOT_FOUND, message=message, detail=None
            ),
            request_id=None,
        )
    )


def raise_chat_function_error(message: str):
    raise ChatFunctionError(
        ErrorResponse(
            error=ErrorResponseError(
                code=ErrorCodes.ERR_CHAT_FUNCTION, message=message, detail=None
            ),
            request_id=None,
        )
    )



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.test_result_enum import TestResultEnum
from ..models.test_status_enum import TestStatusEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.conversation_nested_evaluation import ConversationNestedEvaluation
    from ..models.conversation_nested_turn import ConversationNestedTurn


T = TypeVar("T", bound="ConversationNested")


@_attrs_define
class ConversationNested:
    """
    Attributes:
        id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        status (TestStatusEnum):
        result (TestResultEnum):
        started_at (Union[None, Unset, datetime.datetime]):
        completed_at (Union[None, Unset, datetime.datetime]):
        evaluations (Union[Unset, List['ConversationNestedEvaluation']]):
        conversation_turns (Union[Unset, List['ConversationNestedTurn']]):
    """

    id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    status: TestStatusEnum
    result: TestResultEnum
    started_at: Union[None, Unset, datetime.datetime] = UNSET
    completed_at: Union[None, Unset, datetime.datetime] = UNSET
    evaluations: Union[Unset, List["ConversationNestedEvaluation"]] = UNSET
    conversation_turns: Union[Unset, List["ConversationNestedTurn"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        status = self.status.value

        result = self.result.value

        started_at: Union[None, Unset, str]
        if isinstance(self.started_at, Unset):
            started_at = UNSET
        elif isinstance(self.started_at, datetime.datetime):
            started_at = self.started_at.isoformat()
        else:
            started_at = self.started_at

        completed_at: Union[None, Unset, str]
        if isinstance(self.completed_at, Unset):
            completed_at = UNSET
        elif isinstance(self.completed_at, datetime.datetime):
            completed_at = self.completed_at.isoformat()
        else:
            completed_at = self.completed_at

        evaluations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.evaluations, Unset):
            evaluations = []
            for evaluations_item_data in self.evaluations:
                evaluations_item = evaluations_item_data.to_dict()
                evaluations.append(evaluations_item)

        conversation_turns: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.conversation_turns, Unset):
            conversation_turns = []
            for conversation_turns_item_data in self.conversation_turns:
                conversation_turns_item = conversation_turns_item_data.to_dict()
                conversation_turns.append(conversation_turns_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "created_at": created_at,
                "updated_at": updated_at,
                "status": status,
                "result": result,
            }
        )
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if evaluations is not UNSET:
            field_dict["evaluations"] = evaluations
        if conversation_turns is not UNSET:
            field_dict["conversation_turns"] = conversation_turns

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.conversation_nested_evaluation import ConversationNestedEvaluation
        from ..models.conversation_nested_turn import ConversationNestedTurn

        d = src_dict.copy()
        id = d.pop("id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        status = TestStatusEnum(d.pop("status"))

        result = TestResultEnum(d.pop("result"))

        def _parse_started_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                started_at_type_0 = isoparse(data)

                return started_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        started_at = _parse_started_at(d.pop("started_at", UNSET))

        def _parse_completed_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                completed_at_type_0 = isoparse(data)

                return completed_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        completed_at = _parse_completed_at(d.pop("completed_at", UNSET))

        evaluations = []
        _evaluations = d.pop("evaluations", UNSET)
        for evaluations_item_data in _evaluations or []:
            evaluations_item = ConversationNestedEvaluation.from_dict(evaluations_item_data)

            evaluations.append(evaluations_item)

        conversation_turns = []
        _conversation_turns = d.pop("conversation_turns", UNSET)
        for conversation_turns_item_data in _conversation_turns or []:
            conversation_turns_item = ConversationNestedTurn.from_dict(conversation_turns_item_data)

            conversation_turns.append(conversation_turns_item)

        conversation_nested = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            status=status,
            result=result,
            started_at=started_at,
            completed_at=completed_at,
            evaluations=evaluations,
            conversation_turns=conversation_turns,
        )

        conversation_nested.additional_properties = d
        return conversation_nested

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="APISchemaAgentTargetCreateRequest")


@_attrs_define
class APISchemaAgentTargetCreateRequest:
    """
    Attributes:
        identifier (str):
        role (str):
        name (Union[None, Unset, str]):
        description (Union[None, Unset, str]):
        industry (Union[None, Unset, str]):
        language (Union[None, Unset, str]):  Default: 'en'.
        url (Union[None, Unset, str]):
    """

    identifier: str
    role: str
    name: Union[None, Unset, str] = UNSET
    description: Union[None, Unset, str] = UNSET
    industry: Union[None, Unset, str] = UNSET
    language: Union[None, Unset, str] = "en"
    url: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        identifier = self.identifier

        role = self.role

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        industry: Union[None, Unset, str]
        if isinstance(self.industry, Unset):
            industry = UNSET
        else:
            industry = self.industry

        language: Union[None, Unset, str]
        if isinstance(self.language, Unset):
            language = UNSET
        else:
            language = self.language

        url: Union[None, Unset, str]
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "identifier": identifier,
                "role": role,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if industry is not UNSET:
            field_dict["industry"] = industry
        if language is not UNSET:
            field_dict["language"] = language
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        identifier = d.pop("identifier")

        role = d.pop("role")

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_industry(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        industry = _parse_industry(d.pop("industry", UNSET))

        def _parse_language(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        language = _parse_language(d.pop("language", UNSET))

        def _parse_url(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        url = _parse_url(d.pop("url", UNSET))

        api_schema_agent_target_create_request = cls(
            identifier=identifier,
            role=role,
            name=name,
            description=description,
            industry=industry,
            language=language,
            url=url,
        )

        api_schema_agent_target_create_request.additional_properties = d
        return api_schema_agent_target_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.test_result_enum import TestResultEnum

T = TypeVar("T", bound="APISchemaTestRunResultMetricScores")


@_attrs_define
class APISchemaTestRunResultMetricScores:
    """
    Attributes:
        result (TestResultEnum):
        total (int):
        passed (int):
        failed (int):
        errored (int):
    """

    result: TestResultEnum
    total: int
    passed: int
    failed: int
    errored: int
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        result = self.result.value

        total = self.total

        passed = self.passed

        failed = self.failed

        errored = self.errored

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "result": result,
                "total": total,
                "passed": passed,
                "failed": failed,
                "errored": errored,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        result = TestResultEnum(d.pop("result"))

        total = d.pop("total")

        passed = d.pop("passed")

        failed = d.pop("failed")

        errored = d.pop("errored")

        api_schema_test_run_result_metric_scores = cls(
            result=result,
            total=total,
            passed=passed,
            failed=failed,
            errored=errored,
        )

        api_schema_test_run_result_metric_scores.additional_properties = d
        return api_schema_test_run_result_metric_scores

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.conversation_nested import ConversationNested


T = TypeVar("T", bound="APISchemaConversationTurnCreateResponse")


@_attrs_define
class APISchemaConversationTurnCreateResponse:
    """
    Attributes:
        conversation (ConversationNested):
        turn_id (Union[None, Unset, str]):
    """

    conversation: "ConversationNested"
    turn_id: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        conversation = self.conversation.to_dict()

        turn_id: Union[None, Unset, str]
        if isinstance(self.turn_id, Unset):
            turn_id = UNSET
        else:
            turn_id = self.turn_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "conversation": conversation,
            }
        )
        if turn_id is not UNSET:
            field_dict["turn_id"] = turn_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.conversation_nested import ConversationNested

        d = src_dict.copy()
        conversation = ConversationNested.from_dict(d.pop("conversation"))

        def _parse_turn_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        turn_id = _parse_turn_id(d.pop("turn_id", UNSET))

        api_schema_conversation_turn_create_response = cls(
            conversation=conversation,
            turn_id=turn_id,
        )

        api_schema_conversation_turn_create_response.additional_properties = d
        return api_schema_conversation_turn_create_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
